angular.module('underscore', [])
.factory('_', function() {
  return window._; // assumes underscore has already been loaded on the page
});

var app = angular.module('quoka', ['ionic', 'underscore', 'ionic-material', 'ngCordova', 'ngResource', 'ngTextTruncate', 'ngMaterial','LocalStorageModule', 'ion-sticky', 'chart.js', 'quoka.directives', '$q-spread', 'monospaced.elastic', 'yaru22.angular-timeago', 'ngFileUpload', 'rzModule', 'angAccordion', 'ui.swiper','ionic.contrib.ui.tinderCards2']);

app.run(function ($rootScope, $ionicPlatform, $ionicLoading, Helpers, $cordovaNetwork, Notifications) {
    //attach Helpers factory to rootscope for global access
    $rootScope.helpers = Helpers;

    $ionicPlatform.on("deviceready", function(){
        // Hide the accessory bar by default (remove this to show the accessory bar above the keyboard
        // for form inputs)
        if(window.cordova && window.cordova.plugins.Keyboard) {
          cordova.plugins.Keyboard.hideKeyboardAccessoryBar(true);
        }

        if(window.StatusBar) {
          StatusBar.styleDefault();
        }

        // Check for network connection
        if(window.Connection) {
            if(navigator.connection.type == Connection.NONE) {
                Notifications.showPopup('No Internet Connection', 'Sorry, no Internet connectivity detected. Please reconnect and try again.');
            }
        }

    });
});

app.constant('Config', {
    appVersion: 0.2,
    appBuild: 615,
    apiEndpoint: "https://flauntme.com/m/",
    socialLoginEndpoint: "https://flauntme.com/u/",
    socialShareEndpoint: "https://flauntme.com/m/mobileShare/",
    cache_time_limit: { 
        product: 86400 * 1 * 1000, //2 days in milliseconds
        //product: 1, //2 days in milliseconds
        product_upc: 86400 * 365 * 1000, //1 year milliseconds
        categories: 86400 * 3 * 1000, //3 days in milliseconds
        category_tree: 86400 * 1000, //1 days in milliseconds
        review_extreme: 86400 * 7 * 1000, //7 days in milliseconds
        brand: 86400 * 3 * 1000, 
        brand_products: 86400 * 3 * 1000, 
        brand_followers: 3600 * 6 * 1000, //6 hours
        set: 86400 * 3 * 1000, //3 days
        list: 86400 * 3 * 1000, //3 days
        home: 3600 * 1000, //1 hour        
        //home: 1, //1 hour        
        blog_index: 3600 * 1000 * 2, //2 hours
        blog_post: 3600 * 1000 * 2, //2 hours
        review_temp: 3600 * 1000 * 24 * 90, //90 days
        review_image: 3600 * 1000 * 24 * 90, //90 days
        review_own: 3600 * 1000 * 24 * 30, //30 days
    }, 
    ocr_api_key: 'f87018db3988957',
    subscription_products_android: ['glam_30', 'glam_365'],
    subscription_products_ios: ['com.flauntme.quoka0001.glam_30', 'com.flauntme.quoka0001.glam_365'],
});

/*
    Need to do header modifications to send post to php server using angular
    http://stackoverflow.com/questions/11442632/how-can-i-post-data-as-form-data-instead-of-a-request-payload/
*/
app.config(function ($httpProvider, $ionicConfigProvider) {

    //disable swipe back feature in iOS since some pages are not cached and will show blank pages
    $ionicConfigProvider.views.swipeBackEnabled(false);

    // Intercept POST requests, convert to standard form encoding

    $httpProvider.defaults.headers.post["Content-Type"] = "application/x-www-form-urlencoded";

    $httpProvider.defaults.transformRequest.unshift(function (data, headersGetter) {
        var key, result = [];

        if (typeof data === "string")
            return data;

        for (key in data) {
            if (data.hasOwnProperty(key))
                result.push(encodeURIComponent(key) + "=" + encodeURIComponent(data[key]));
        }

        return result.join("&");
    });

    $httpProvider.interceptors.push('APIInterceptor');
});


app.config(function($stateProvider, $urlRouterProvider) {
    $stateProvider

    //LOGIN/REGISTER/TOUR

    .state('root', {
        url: "/",
        template: "",
        controller: 'RootCtrl'
    })

    .state('auth', {
        url: "/auth",
        templateUrl: "templates/auth/auth.html",
        abstract: true,
        controller: 'AuthCtrl'
    })

    .state('auth.login', {
        url: '/login',
        templateUrl: "templates/auth/login.html",
        controller: 'LoginCtrl'
    })    

    .state('auth.signup', {
        url: '/signup',
        templateUrl: "templates/auth/signup.html",
        controller: 'SignupCtrl'
    })

    .state('auth.forgot-password', {
        url: "/forgot-password",
        templateUrl: "templates/auth/forgot-password.html",
        controller: 'ForgotPasswordCtrl'
    })

    .state('auth.tour', {
        url: "/tour",
        templateUrl: "templates/tour.html",
        controller: 'TourController'
    })

    .state('auth.terms', {
        url: "/terms",
        templateUrl: "templates/terms.html",
        controller: 'TermsController'
    })

    .state('app', {
        url: '/app',
        abstract: true,
        templateUrl: 'templates/menu.html',
        controller: 'AppCtrl'
    })    

    .state('app.product', {
        url: "/product/:productId",
        views: {
            'menuContent': {
            templateUrl: "templates/product.html",
            controller: 'ProductCtrl'
            }
        },
        resolve: {
            view_data: function(ProductService, $stateParams) 
            {
                return ProductService.getProductByID($stateParams.productId);               
            }
        }
    })

    .state('app.productByUPC', {
        url: "/productByUPC/:productUPC",
        views: {
            'menuContent': {
            templateUrl: "templates/product.html",
            controller: 'ProductCtrl'
            }
        },
        resolve: {
            view_data: function(ProductService, $stateParams) 
            {
                return ProductService.getProductByUPC($stateParams.productUPC);
            }
        }
    })

    .state('app.barcodeNotFound', {
        url: "/barcodeNotFound/:productUPC",
        views: {
            'menuContent': {
            templateUrl: "templates/barcode_not_found.html",
            controller: 'barcodeNotFoundController'
            }
        }
    })

    .state('app.reviewWrite', {
        url: "/reviewWrite/:productId",
        views: {
            'menuContent': {
            templateUrl: "templates/review-write.html",
            controller: 'reviewWriteController'
            }
        }
    })

    .state('app.brand', {
        url: "/brand/:brandId",
        views: {
            'menuContent': {
            templateUrl: "templates/brand.html",
            controller: 'BrandCtrl'
            }
        },
        resolve: {
            view_data: function(BrandService, $stateParams) 
            {
                return BrandService.brandByID($stateParams.brandId);             
            }
        }
    })

    .state('app.brandProducts', {
        url: "/brandProducts/:brandId",
        views: {
            'menuContent': {
            templateUrl: "templates/brand_products.html",
            controller: 'BrandProductsCtrl'
            }
        },
        resolve: {
            brand_data: function(BrandService, $stateParams) 
            {
                return BrandService.brandByID($stateParams.brandId);             
            }
        }
    })

    .state('app.brandFollowers', {
        url: "/brandFollowers/:brandId",
        cache: false,
        views: {
            'menuContent': {
            templateUrl: "templates/brand_followers.html",
            controller: 'BrandFollowersCtrl'
            }
        },
        resolve: {
            view_data: function(BrandService, $stateParams) 
            {
                return BrandService.getFollowers($stateParams.brandId);             
            }
        }
    })

    .state('app.category', {
        url: "/category/:categoryId",
        views: {
            'menuContent': {
            templateUrl: "templates/category.html",
            controller: 'CategoryCtrl'
            }
        }
    })

    .state('app.categoryTree', {
        url: "/categoryTree",
        views: {
            'menuContent': {
            templateUrl: "templates/partials/categorylist.html",
            controller: 'categoryListController'
            }
        },        
    })

    .state('app.review', {
        url: "/review/:reviewId",
        views: {
            'menuContent': {
            templateUrl: "templates/product_reviews.html",
            controller: 'ReviewProductController'
            }
        },
        resolve: {
            ProductID: function($stateParams) 
            {
                return $stateParams.reviewId;             
            }
        }
    })

    .state('app.reviewInternal', {
        url: "/reviewInternal/:productId",
        views: {
            'menuContent': {
            templateUrl: "templates/product-reviews-user.html",
            controller: 'UserReviewProductController'
            }
        }
    })

    .state('app.list', {
        url: "/list/:listId",
        views: {
            'menuContent': {
            templateUrl: "templates/list.html",
            controller: 'ListController'
            }
        },
    })

    .state('app.set', {
        url: "/set/:setId",
        views: {
            'menuContent': {
            templateUrl: "templates/set.html",
            controller: 'SetCtrl'
            }
        },
        resolve: {
            view_data: function(ProductService, $stateParams) 
            {
                return ProductService.getSet($stateParams.setId);             
            }
        }
    })

    .state('app.recentProduct', {
        url: "/recentProduct",
        cache: false,
        views: {
            'menuContent': {
            templateUrl: "templates/products_recent.html",
            controller: 'ProductRecentCtrl'
            }
        },
        resolve: {
            view_data: function(cacheJSON) 
            {                
                return cacheJSON.getProductsAll();             
            }
        }
    })

    .state('app.search', {
        url: '/search',
        views: {
            'menuContent': {
                templateUrl: 'templates/search.html',
                controller: 'SearchController'
            }
        }
    })

    .state('app.user', {
        url: "/user/:username",
        views: {
            'menuContent': {
            templateUrl: 'templates/user.html',
                controller: 'UserController'
            }
        }
    })

    .state('app.userFollowers', {
        url: "/userFollowers/:username",
        views: {
            'menuContent': {
            templateUrl: 'templates/user-followers.html',
                controller: 'UserFollowersController'
            }
        }
    })

    .state('app.userFollowing', {
        url: "/userFollowing/:username",
        views: {
            'menuContent': {
            templateUrl: 'templates/user-following.html',
                controller: 'UserFollowingController'
            }
        }
    })

    .state('app.userRatings', {
        url: "/userRatings/:username",
        views: {
            'menuContent': {
            templateUrl: 'templates/user-ratings.html',
                controller: 'UserRatingsController'
            }
        }
    })

    .state('app.userReview', {
        url: "/userReview/:reviewID",
        views: {
            'menuContent': {
            templateUrl: 'templates/user-review.html',
                controller: 'UserReviewController'
            }
        }
    })

    .state('app.account', {
        url: "/account",
        views: {
            'menuContent': {
            templateUrl: 'templates/account.html',
                controller: 'AccountController'
            }
        }
    })

    .state('app.accountEmail', {
        url: "/accountEmail",
        views: {
            'menuContent': {
            templateUrl: 'templates/account-email.html',
                controller: 'AccountEmailController'
            }
        }
    })

    .state('app.accountPassword', {
        url: "/accountPassword",
        views: {
            'menuContent': {
            templateUrl: 'templates/account-password.html',
                controller: 'AccountPasswordController'
            }
        }
    })

    .state('app.blog', {
        url: "/blog",
        views: {
            'menuContent': {
            templateUrl: 'templates/blog_index.html',
                controller: 'BlogIndexController'
            }
        },
        resolve: {
            view_data: function(BlogService) 
            {                
                return BlogService.getBlogIndex();             
            }
        }
    })

    .state('app.blogPost', {
        url: "/blogPost/:postId",
        views: {
            'menuContent': {
            templateUrl: "templates/blog_post.html",
            controller: 'BlogPostController'
            }
        },
        resolve: {
            view_data: function(BlogService, $stateParams) 
            {
                return BlogService.getPost($stateParams.postId);             
            }
        }
    })

    .state('app.assistant', {
        url: '/assistant',
        views: {
            'menuContent': {
                templateUrl: 'templates/assistant.html',
                controller: 'AssistantController'
            }
        }
    })

    .state('app.assistantWizard', {
        url: '/assistantWizard',
        views: {
            'menuContent': {
                templateUrl: 'templates/assistant-wizard.html',
                controller: 'AssistantWizardController'
            }
        }
    })

    .state('app.assistantAdvance', {
        url: '/assistantAdvance',
        views: {
            'menuContent': {
                templateUrl: 'templates/assistant-advance.html',
                controller: 'AssistantAdvanceController'
            }
        }
    })

    .state('app.assistantRecs', {
        url: '/assistantRecs',
        views: {
            'menuContent': {
                templateUrl: 'templates/assistant-recommendations.html',
                controller: 'AssistantRecommendationsController'
            }
        }
    })

    .state('app.assistantHistory', {
        url: '/assistantHistory',
        views: {
            'menuContent': {
                templateUrl: 'templates/assistant-history.html',
                controller: 'AssistantHistoryController'
            }
        }
    })

    .state('app.finder', {
        url: '/finder',
        views: {
            'menuContent': {
                templateUrl: 'templates/finder.html',
                controller: 'FinderController'
            }
        }
    })

    .state('app.help', {
        url: '/help',
        views: {
            'menuContent': {
                templateUrl: 'templates/help.html',
                controller: 'HelpController'
            }
        }
    })

    .state('app.settings', {
        url: '/settings',
        views: {
            'menuContent': {
                templateUrl: 'templates/settings.html',
                controller: 'SettingsController'
            }
        }
    })

    .state('app.glam', {
        url: '/glam',
        views: {
            'menuContent': {
                templateUrl: 'templates/glam.html',
                controller: 'GlamController'
            }
        }
    })    

    .state('app.ocr', {
        url: '/ocr',
        cache: false,
        views: {
            'menuContent': {
                templateUrl: 'templates/ocr.html',
                controller: 'OCRController'
            }
        }
    })

    .state('app.home', {
        url: '/home',
        cache: false,
        views: {
            'menuContent': {
                templateUrl: 'templates/home.html',
                controller: 'HomeController'
            }
        }
    })

    ;
    
    // if none of the above states are matched, use this as the fallback
    $urlRouterProvider.otherwise('/');    
});

