angular.module('quoka.directives', [])

.directive('myTabs', function() {
	return {
		restrict: 'E',
		transclude: true,
		scope: {},
		controller: function($scope) {
			var tabs = $scope.tabs = [];

			$scope.select = function(tab) {
				angular.forEach(tabs, function(tab) {
					tab.selected = false;
				});
				tab.selected = true;
				$scope.$emit('my-tabs-changed', tab);
			};

			this.addTab = function(tab) {
				if (tabs.length === 0) {
					$scope.select(tab);
				}
				tabs.push(tab);
			};
		},
		templateUrl: 'templates/common/my-tabs.html'
	};
})

.directive('myTab', function() {
	return {
		require: '^myTabs',
		restrict: 'E',
		transclude: true,
		scope: {
			title: '@'
		},
		link: function(scope, element, attrs, tabsCtrl) {
			tabsCtrl.addTab(scope);
		},
		templateUrl: 'templates/common/my-tab.html'
	};
})

.directive('validPin', function($http) {
	return {
		require: 'ngModel',
		link: function(scope, ele, attrs, c) {
			scope.$watch(attrs.ngModel, function(pinValue) {
				// $http({
				// 	method: 'POST',
				// 	url: '/api/check/' + attrs.validPin,
				// 	data: {'pin': attrs.validPin}
				// }).success(function(data, status, headers, cfg) {
				// 	c.$setValidity('valid-pin', data.isValid);
				// }).error(function(data, status, headers, cfg) {
				// 	c.$setValidity('valid-pin', false);
				// });
				if(pinValue=="12345")
				{
					c.$setValidity('valid-pin', true);
				}
				else
				{
					c.$setValidity('valid-pin', false);
				}
			});
		}
	};
})


.directive('showHideContainer', function(){
	return {
		scope: {
		},
		controller: function($scope, $element, $attrs) {
			$scope.show = false;

			$scope.toggleType = function($event){
				$event.stopPropagation();
				$event.preventDefault();

				$scope.show = !$scope.show;

				// Emit event
				$scope.$broadcast("toggle-type", $scope.show);
			};
		},
		templateUrl: 'templates/common/show-hide-password.html',
		restrict: 'A',
		replace: false,
		transclude: true
	};
})


.directive('showHideInput', function(){
	return {
		scope: {
		},
		link: function(scope, element, attrs) {
			// listen to event
			scope.$on("toggle-type", function(event, show){
				var password_input = element[0],
						input_type = password_input.getAttribute('type');

				if(!show)
				{
					password_input.setAttribute('type', 'password');
				}

				if(show)
				{
					password_input.setAttribute('type', 'text');
				}
			});
		},
		require: '^showHideContainer',
		restrict: 'A',
		replace: false,
		transclude: false
	};
})


.directive('biggerText', function($ionicGesture) {
	return {
		restrict: 'A',
		link: function(scope, element, attrs) {
			$ionicGesture.on('touch', function(event){
				event.stopPropagation();
				event.preventDefault();

				var text_element = document.querySelector(attrs.biggerText),
						root_element = document.querySelector(".menu-content"),
						current_size_str = window.getComputedStyle(text_element, null).getPropertyValue('font-size'),
						current_size = parseFloat(current_size_str),
						new_size = Math.min((current_size+2), 24),
						new_size_str = new_size + 'px';

				root_element.classList.remove("post-size-"+current_size_str);
				root_element.classList.add("post-size-"+new_size_str);
			}, element);
		}
	};
})

.directive('smallerText', function($ionicGesture) {
	return {
		restrict: 'A',
		link: function(scope, element, attrs) {
			$ionicGesture.on('touch', function(event){
				event.stopPropagation();
				event.preventDefault();

				var text_element = document.querySelector(attrs.smallerText),
				root_element = document.querySelector(".menu-content"),
				current_size_str = window.getComputedStyle(text_element, null).getPropertyValue('font-size'),
				current_size = parseFloat(current_size_str),
				new_size = Math.max((current_size-2), 12),
				new_size_str = new_size + 'px';

				root_element.classList.remove("post-size-"+current_size_str);
				root_element.classList.add("post-size-"+new_size_str);
			}, element);
		}
	};
})

.directive('ionicYoutubeVideo', function($timeout, $ionicPlatform, youtubeEmbedUtils) {
	return {
		restrict: 'E',
		scope: {
			videoId: '@'
		},
		controller: function($scope, $element, $attrs) {
			$scope.playerVars = {
				rel: 0,
				showinfo: 0
			};
			$ionicPlatform.on("pause", function(){
				var yt_ready = youtubeEmbedUtils.ready;
				if(yt_ready)
				{
					$scope.yt_video.stopVideo();
				}
		  });
		},
		templateUrl: 'templates/common/ionic-youtube-video.html',
		replace: false
	};
})

.directive('postContent', function($timeout, _, $compile) {
	return {
		restrict: 'A',
		scope: {},
		link: function(scope, element, attrs) {
			/**
			 * JavaScript function to match (and return) the video Id
			 * of any valid Youtube Url, given as input string.
			 * @author: Stephan Schmitz <eyecatchup@gmail.com>
			 * @url: http://stackoverflow.com/a/10315969/624466
			 */
			//  Ver: https://regex101.com/r/tY9jN6/1
			function ytVidId(url) {
			  var p = /^(?:https?:\/\/)?(?:www\.)?(?:youtu\.be\/|youtube\.com\/(?:embed\/|v\/|watch\?v=|watch\?.+&v=))((\w|-){11,})(?:\S+)?$/gmi;
			  return (url.match(p)) ? RegExp.$1 : false;
			}

			$timeout(function(){
				var iframes = element.find('iframe');
				if(iframes.length > 0)
				{
					angular.forEach(iframes, function(i) {

						var iframe = angular.element(i),
								youtube_video_id = ((iframe.length > 0) && (!_.isUndefined(iframe[0].src))) ? ytVidId(iframe[0].src) : false;
						if(youtube_video_id !== false)
						{
							// Then it's a youtube video, compile our custom directive
							var ionic_yt_video = $compile("<ionic-youtube-video video-id='"+youtube_video_id+"'></ionic-youtube-video>")(scope);
        			iframe.parent().append(ionic_yt_video);
							iframe.remove();
						}
					});
				}
			}, 10);
		}
	};
})

//Use this directive to open external links using inAppBrowser cordova plugin
.directive('dynamicAnchorFix', function($ionicGesture, $timeout, $cordovaInAppBrowser) {
	return {
		scope: {},
		link: function(scope, element, attrs) {
			$timeout(function(){
				var anchors = element.find('a');
				if(anchors.length > 0)
				{
					angular.forEach(anchors, function(a) {

						var anchor = angular.element(a);

						anchor.bind('click', function (event) {
							event.preventDefault();
							event.stopPropagation();

							var href = event.currentTarget.href;
							var	options = {};

							//inAppBrowser see documentation here: http://ngcordova.com/docs/plugins/inAppBrowser/

							$cordovaInAppBrowser.open(href, '_blank', options)
								.then(function(e) {
									// success
								})
								.catch(function(e) {
									// error
								});
						});

					});
				}
			}, 10);
		},
		restrict: 'A',
		replace: false,
		transclude: false
	};
})


.directive('multiBg', function(_){
	return {
		scope: {
			multiBg: '=',
			interval: '=',
			helperClass: '@'
		},
		controller: function($scope, $element, $attrs, $interval) {
			$scope.loaded = false;
			var utils = this;

			this.animateBg = function(){
				// Think i have to use apply because this function is not called from this controller ($scope)
				$scope.$apply(function () {
					$scope.loaded = true;
					$element.css({'background-image': 'url(' + $scope.bg_img + ')'});
				});
			};

			this.setBackground = function(bg) {
				$scope.bg_img = bg;
			};

			if(!_.isUndefined($scope.multiBg))
			{ 
				if(_.isArray($scope.multiBg) && ($scope.multiBg.length > 1) && !_.isUndefined($scope.interval) && _.isNumber($scope.interval))
				{				
					// Then we need to loop through the bg images

					function updateBG(i) {
				        var j=(i+1)%($scope.multiBg.length); //(i+1) to start at second image
				        //so the j rotates like: 1,2,3,4,0,1,2,...

				        utils.setBackground($scope.multiBg[j]);
				    }

				    utils.setBackground($scope.multiBg[0]);
				    stopWord = $interval(updateBG, $scope.interval, 20);//executes 'updateWord' every second
				}
				else
				{
					// Then just set the multiBg image as background image
					utils.setBackground($scope.multiBg[0]);
				}
			}
		},
		templateUrl: 'templates/common/multi-bg.html',
		restrict: 'A',
		replace: true,
		transclude: true
	};
})


.directive('bg', function() {
	return {
		restrict: 'A',
		require: '^multiBg',
		scope: {
			ngSrc: '@'
		},
		link: function(scope, element, attr, multiBgController) {
			element.on('load', function() {
				multiBgController.animateBg();
		  });
		}
	};
})

.directive('preImg', function() {
	return {
		restrict: 'E',
		transclude: true,
		scope: {
			ratio:'@',
			helperClass: '@'
		},
		controller: function($scope) {
			$scope.loaded = false;

			this.hideSpinner = function(){
				// Think i have to use apply because this function is not called from this controller ($scope)
				$scope.$apply(function () {
					$scope.loaded = true;
				});
			};
		},
		templateUrl: 'templates/common/pre-img.html'
	};
})

.directive('backgroundImg', function(){
    return function(scope, element, attrs){
        attrs.$observe('backgroundImg', function(value) {
            element.css({
                'background-image': 'url(' + value +')'
            });
        });
    };
})

.directive('spinnerOnLoad', function() {
	return {
		restrict: 'A',
		require: '^preImg',
		scope: {
			ngSrc: '@'
		},
		link: function(scope, element, attr, preImgController) {
			element.on('load', function() {
				preImgController.hideSpinner();
		  });
		}
	};
})

.directive('userListSmall', function() {
	return {
		restrict: 'E',
		replace: true,
		templateUrl: 'templates/partials/userlistsmall.html',
		transclude: true,
		link: function(scope, element, attrs) {
			scope.sortOrder = scope.sort === undefined ? 'id' : scope.sort;	
		
        }
	};
})

.directive('productList', function(GlobalSettingsFactory) {
	return {
		restrict: 'E',
		replace: true,
		templateUrl: 'templates/partials/product-display-grid.html',
		transclude: true,
		link: function(scope, element, attrs) {
			//set defaults
			if (scope.sortOrder === undefined)
				scope.sortOrder = 'relevance';

			if (scope.sortDesc === undefined)		
				scope.sortDesc = true;		

			scope.settings = GlobalSettingsFactory.getCache();		
        }
	};
})

.directive('productDisplayList', function(GlobalSettingsFactory) {
	return {
		restrict: 'E',
		replace: true,
		templateUrl: 'templates/partials/product-display-list.html',
		transclude: true,
		scope: {
			items: "=",
			sort: "@",
			desc: "@",
			limit: "@",
		},
		link: function(scope, element, attrs) {
			scope.settings = GlobalSettingsFactory.getCache();			
        }
	};
})

.directive('productListMinimal', function() {
	return {
		restrict: 'E',
		templateUrl: 'templates/partials/productlistminimal.html',
		transclude: true
	};
})

.directive('reviewExternalList', function(ReviewFinderFactory) {
	return {
		restrict: 'E',
		replace: true,
		templateUrl: 'templates/partials/review-external.html',
		transclude: true,
		link: function(scope, element, attrs) {
			

        }
	};
})

.directive('reviewInternalList', function() {
	return {
		restrict: 'E',
		replace: true,
		templateUrl: 'templates/partials/review-internal.html',
		transclude: true,
		link: function(scope, element, attrs) {
						
        }
	};
})

.directive('heartAction', function(HeartService) {
	return {
		restrict: 'E',
		replace: true,
		templateUrl: 'templates/partials/heart-action.html',
		scope: {
			hearted: '@',
			mid: '@'
		},
		link: function(scope, element, attrs) {
			scope.isHearted = scope.hearted === '1' ? true : false;

			scope.addHeart = function (){

				HeartService.toggleHeart(scope.mid).then(function(success){
					//success
					scope.isHearted = !scope.isHearted;
				}, function(error){
					console.log(error);
					//error

				});
			};
        }
	};
})

.directive('ratingDisplayList', function() {
	return {
		restrict: 'E',
		replace: true,
		templateUrl: 'templates/partials/rating-display-list.html',
		transclude: true,
		scope: {
			ratings: "=",
			sort: "@",
			desc: "@",
			limit: "@",
		},
		link: function(scope, element, attrs) {
			
        }
	};
})

.directive('userDisplayList', function() {
	return {
		restrict: 'E',
		replace: true,
		templateUrl: 'templates/partials/user-display-list.html',
		transclude: true,
		scope: {
			users: "=",
			sort: "@",
			desc: "@",
			limit: "@",
		},
		link: function(scope, element, attrs) {
			
        }
	};
})

.directive('followAction', function(BrandService, $cordovaToast) {
	return {
		restrict: 'E',
		replace: true,
		templateUrl: 'templates/partials/brand-action.html',
		link: function(scope, element, attrs) {
			console.log(scope.brand);
			console.log(scope.brand.follow_state);
			scope.isFollowed = scope.brand.follow_state === 'Unfollow' ? true : false;
			console.log(scope.isFollowed);
			scope.toggleFollow = function (){

				BrandService.toggleFollow(scope.brand.id)
				.then(function(success){
					//success
					scope.isFollowed = !scope.isFollowed;

					if (scope.brand.follow_state === 'Unfollow')
					{
						scope.brand.follow_state = 'Follow';
						scope.brand.followers--;
					}
					else
					{
						scope.brand.follow_state = 'Unfollow';
						scope.brand.followers++;
					}					

					$cordovaToast.showLongBottom(success.message);
				}, function(error){
					console.log(error);
					//error

				});
			};
        }
	};
})

.directive('userFollow', function(UserService, $cordovaToast) {
	return {
		restrict: 'E',
		replace: true,
		templateUrl: 'templates/partials/user-follow.html',
		scope: { state: "@"},
		link: function(scope, element, attrs) {

			console.log(scope.state);
			console.log(scope.user);
			console.log(scope.user.follow_state);
			scope.isFollowed = scope.user.follow_state === 'Unfollow' ? true : false;

			scope.toggleFollow = function (){

				UserService.toggleFollow(scope.user.username)
				.then(function(success){
					//success
					scope.isFollowed = !scope.isFollowed;

					if (scope.user.follow_state === 'Unfollow')
					{
						scope.user.follow_state = 'Follow';
						scope.user.followers--;
					}
					else
					{
						scope.user.follow_state = 'Unfollow';
						scope.user.followers++;
					}					

					$cordovaToast.showLongBottom(success.message);
				}, function(error){
					console.log(error);
					//error

				});
			};
        }
	};
})

.directive('homeContentAssistant', function() {
	return {
		restrict: 'E',
		replace: false,
		templateUrl: 'templates/partials/home-content-assistant.html',
		transclude: true,
		link: function(scope, element, attrs) {
				
		
        }
	};
})

.directive('homeContentPost', function() {
	return {
		restrict: 'E',
		replace: false,
		templateUrl: 'templates/partials/home-content-post.html',
		transclude: true,
		link: function(scope, element, attrs) {
				
		
        }
	};
})

.directive('homeContentProduct', function() {
	return {
		restrict: 'E',
		replace: false,
		templateUrl: 'templates/partials/home-content-product.html',
		transclude: true,
		link: function(scope, element, attrs) {
				
		
        }
	};
})

.directive('homeContentList', function() {
	return {
		restrict: 'E',
		replace: false,
		templateUrl: 'templates/partials/home-content-list.html',
		transclude: true,
		link: function(scope, element, attrs) {
				
		
        }
	};
})

.directive('homeContentInstagram', function() {
	return {
		restrict: 'E',
		replace: false,
		templateUrl: 'templates/partials/home-content-instagram.html',
		transclude: true,
		link: function(scope, element, attrs) {
				
		
        }
	};
})

.directive('homeContentNotice', function() {
	return {
		restrict: 'E',
		replace: false,
		templateUrl: 'templates/partials/home-content-notice.html',
		transclude: true,
		link: function(scope, element, attrs) {
				
		
        }
	};
})

.directive('defaultAvatar', function() {
	return {
		restrict: 'E',
		replace: true,
		templateUrl: 'templates/partials/default-avatar.html',
		scope: {
			uid: '@',
			size: '@'
		},
		link: function(scope, element, attrs) {
			var rand = scope.uid % 10,
				width = typeof scope.size !== 'undefined' ? parseInt(scope.size) : 65,
				hpostion = rand * width * -1;

			scope.avatarInfo = {background: 'url("img/default/avatarsprites.jpg") no-repeat '+hpostion+'px 0px / auto '+width+'px'};
        },
     
	};
})

.directive('ratingStar', function() {
    return {
	    restrict: 'E',
	    transclude: true,
	    templateUrl: 'templates/partials/rating-star.html',
	    scope: {
	        selectedText: '=',
	        userRating: '='
	    },
	    link: function(scope, element, attrs) {	  
	    	      
	    	scope.possible = {
	    		1: {value: 1, text: 'Horrible', selected: false},
	    		2: {value: 2, text: 'Terrible', selected: false},
	    		3: {value: 3, text: 'Bad', selected: false},
	    		4: {value: 4, text: 'Poor', selected: false},
	    		5: {value: 5, text: 'Mediocre', selected: false},
	    		6: {value: 6, text: 'Fair', selected: false},
	    		7: {value: 7, text: 'Good', selected: false},
	    		8: {value: 8, text: 'Great', selected: false},
	    		9: {value: 9, text: 'Superb', selected: false},
	    		10: {value: 10, text: 'Perfect', selected: false},
    		};

	        //Called when he user clicks on the rating
	        scope.setValue = function(val) {	        	
	          	scope.userRating = val;	

	          	//update display text
	          	scope.selectedText = scope.possible[val].text;
	          	
	          	//de-select all classes
	          	angular.forEach(scope.possible, function(value, key) {
	          		scope.possible[key].selected = false;
				});
	          	//add select to clicked class
	          	scope.possible[val].selected = true;	          	
	        };
	    }
    }; 
})

.directive('commentBox', function(CommentService, Notifications, $cordovaToast) {
	return {
		restrict: 'E',
	    transclude: true,
	    templateUrl: 'templates/partials/comment-box.html',
	    link: function(scope, element, attrs) {	
	    	//used to disable form after submit
			scope.hasSubmitted = true;

			//resizes the footer when the comment gets longer
			scope.$on('elastic:resize', function(event, element, oldHeight, newHeight) {
				var	footer = document.getElementById("commentFooter");

    			footer.style.height = 26 + newHeight + "px";
			});

	    	scope.submit = function (){
	    		var data = scope.data;
	    		
	    		//verify/error check
	    		if (angular.isUndefined(data.comment) || data.comment === '')
	    		{
	    			Notifications.showPopup('Alert', "Cannot submit a blank comment.");
	    			return;
	    		}

	    		if (data.comment.length <= 5)
	    		{
	    			Notifications.showPopup('Alert', "Your comment is too short.");
	    			return;
	    		}

	    		if (data.comment.length > 2500)
	    		{
	    			Notifications.showPopup('Alert', "Your comment is too long.");
	    			return;
	    		}

	    		scope.hasSubmitted = false;
	    			
				CommentService.submit(data).then(function(success){
					//success
					
					//clear text field
					scope.data.comment = '';

					//let user know everything was ok
					$cordovaToast.showLongBottom(success.message);

					//open comment view
					scope.openComments();

				}, function(error){
					console.log(error);
					//error

				});
			};
	    }
	};
})

.directive('backbutton', function() {
    return {
        restrict: 'E',
        replace: true,
        scope: false,
        template: '<button class="button button-icon button-clear current"><i class="c-black icon"></i></button>',
        compile: function (element, attrs) {
            var icon = ionic.Platform.isIOS() ? 'ion-ios-arrow-back' : 'ion-android-arrow-back';
            angular.element(element[0]).children().addClass(icon);
        }
    };
 })

.directive('menuRoot', function() {
	return {
		restrict: 'E',
		templateUrl: 'templates/partials/menuRoot.html',
		transclude: true,
		replace: true,
	};
})

;


