app.factory('Helpers', function($cordovaInAppBrowser){
	return {
	    getProductLink: function(id) {
	      	return '#/app/product/'+id;
	    },
	    getBrandLink: function(id) {
	      	return '#/app/brand/'+id;
	    },
	    getBrandProductsLink: function(id) {
	      	return '#/app/brandProducts/'+id;
	    },
	    getBrandFollowersLink: function(id) {
	      	return '#/app/brandFollowers/'+id;
	    },
	    getCategoryLink: function(id) {
	      	return '#/app/category/'+id;
	    },
	    getReviewLink: function(id) {
	      	return '#/app/review/'+id;
	    },
	    getListLink: function(id) {
	      	return '#/app/list/'+id;
	    },
	    getSetLink: function(id) {
	      	return '#/app/set/'+id;
	    },
	    getBlogLink: function(id) {
	      	return '#/app/blogPost/'+id;
	    },
	    getUserLink: function(id) {
	      	return '#/app/user/'+id;
	    },
	    externalLink: function(url) {
		      	var options = {
	      		location: 'yes',
	      		clearcache: 'yes',
	      		toolbar: 'no'
	   		};

	        $cordovaInAppBrowser.open(url, '_system', options)		
	      	.then(function(event) {
	        	// success
	      	})		
	      	.catch(function(event) {
	        	// error
	      	});
	    },
	    //http://stackoverflow.com/questions/19655975/check-if-an-array-contains-duplicate-values
		checkDuplicate: function(arr) {
		    var map = {}, i, size;

		    for (i = 0, size = arr.length; i < size; i++){
		        if (map[arr[i]]){
		            return false;
		        }

		        map[arr[i]] = true;
		    }

		    return true;
		},
		//http://stackoverflow.com/questions/9804777/how-to-test-if-a-string-is-json-or-not
		isJSON: function(str){
		    try {
		        var MyJSON = JSON.stringify(str);
		        var json = JSON.parse(MyJSON);
		        if(typeof(str) === 'string')
		            if(str.length === 0)
		                return false;
		    }
		    catch(e){
		        return false;
		    }
		    return true;
		}
	};
})

.factory('searchHistory', function(localStorageService) {

	var key = 'cache_search_history',
		obj = { 'beauty': [], 'users': [] };

	return {
		get: function() {
	    	var cache = localStorageService.get(key);				
           	return cache;	
	    },
		save: function(set, text) {

			//has to be at least 3 chars
			if (text.length <= 2)
				return this.get();

			//get product cache
			var cache = localStorageService.get(key);

	    	//does cache exist?
			if (!angular.isObject(cache))
				//no cache set, probably new install
				cache = obj;

			//check if text already exist in cache
			if (cache[set].indexOf(text) > -1)
				//exist, no need to add
				return this.get();		
			
			//add new text to cache array
			cache[set].push(text);		

			//trim to five items if necessary
			if (cache[set].length > 5)
				cache[set] = cache[set].slice(-5);

			//save new cache
			localStorageService.set(key, cache);

		   	return this.get();
	    }
	};
})

.factory('cacheJSON', function(Config, localStorageService){

	return {
		addProduct: function(product) {
			console.log('addProductToCache');
			//get product cache
			var cache = localStorageService.get('cache_product');

			if (!angular.isObject(cache))
				//no cache set, probably new install
				cache = {};

			//check if product already exist in cache
			if (angular.isDefined(cache[product.id]))	
				return true;
			console.log('000');
			//add timestamp
			product.cache_time = Date.now();
			
			//add new product to cache
			cache[product.id] = product;			

			//save new cache
			localStorageService.set('cache_product', cache);

		   	return true;
	    },
	    getProduct: function(productID) {
	    	console.log('getProduct');
	    	var cache = localStorageService.get('cache_product');
			
	    	if (!angular.isObject(cache))
	    		//no cache set
	    		return false;
	    	//console.log('1111111111');	
	    	if (!angular.isDefined(cache[productID]))
	    		//cache exist, but not the specific product
	    		return false;
	    	//console.log('2222222222222');	
	    	//cache exist, check age
	    	var product = cache[productID];
	    	var currentTime = Date.now();

	    	//is current time greater than data cache time + config.cache_limit ?
	    	//note that js time is in milliseconds
	    	if (currentTime > product.cache_time + Config.cache_time_limit.product)
                //too old, grab product info again
           		return false;	
           	//console.log('3333333');
           	return product;	
	    },
	    getProductsAll: function() {
	    	console.log('getProductsAll');
	    	var cache = localStorageService.get('cache_product');
			
	    	if (!angular.isObject(cache))
	    		//no cache set
	    		return false;

           	return cache;	
	    },
	    getCache: function(id, type) {
	    	var key = 'cache_'+type,
	    		cache = localStorageService.get(key);
			
	    	if (!angular.isObject(cache))
	    		//no cache set
	    		return false;

	    	if (!angular.isDefined(cache[id]))
	    		//cache exist, but not the specific set
	    		return false;

	    	//cache exist, check age
	    	var data = cache[id],
	    		currentTime = Date.now();

	    	//is current time greater than data cache time + config.cache_limit ?
	    	//note that js time is in milliseconds
	    	if (currentTime > data.cache_time + Config.cache_time_limit[type])
                //too old, grab set info again
           		return false;	

           	return data;	
	    },
	    setCache: function(id, json, type, replace) {
			//get product cache
			var key = 'cache_'+type,
	    		cache = localStorageService.get(key),
	    		replace = typeof replace !== 'undefined' ? true : false;

			if (!angular.isObject(cache))
				//no cache set, probably new install
				cache = {};

			//check if product already exist in cache
			if (angular.isDefined(cache[id]))
				//it does, are we allowed to replace?
				if (!replace)	
					//no, so don't proceed
					return true;		
			
			//add new product to cache
			cache[id] = json;	

			//add timestamp
			cache[id].cache_time = Date.now();		

			//save new cache
			localStorageService.set(key, cache);

		   	return true;
	    },
	    unsetCache: function(id, type) {
	    	var key = 'cache_'+type,
	    		cache = localStorageService.get(key);
			
	    	if (!angular.isObject(cache))
	    		//no cache set
	    		return false;

	    	if (!angular.isDefined(cache[id]))
	    		//cache exist, but not the specific set
	    		return false;

	    	//delete
	    	delete cache[id];

	    	//save new cache
	    	localStorageService.set(key, cache);	

           	return true;	
	    },
	};
})

//https://devdactic.com/complete-image-guide-ionic/
.factory('ImageService', function($cordovaCamera, $q, $cordovaFile) {	
 
  	function makeid() {
	    var text = '';
	    var possible = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789';
	 
	    for (var i = 0; i < 5; i++) {
	      	text += possible.charAt(Math.floor(Math.random() * possible.length));
	    }

	    return text;
  	}
 
  	function optionsForType(type) {
	    var cam;

	    switch (type) {
	      	case 0:
		        cam = 0; //back
		        break;
		    case 1:
		        cam = 1; //front
		        break;    
	    }

	    return {
	      	destinationType: Camera.DestinationType.FILE_URI,
	      	sourceType: Camera.PictureSourceType.CAMERA,
	      	allowEdit: false,
	      	encodingType: Camera.EncodingType.JPEG,
	      	popoverOptions: CameraPopoverOptions,
	      	targetWidth: 1200,
	      	saveToPhotoAlbum: false,
	      	correctOrientation: true,
	      	cameraDirection: cam
	    };
  	}

  	/*function hasReadPermission() {
  		console.log('hasReadPermission');
    	window.imagePicker.hasReadPermission(
      		function(result) {
        		// if this is 'false' you probably want to call 'requestReadPermission' now
        		console.log(result);
        		if (!result)
        			requestReadPermission();
      		}
    	);
  	}

  	function requestReadPermission() {
    	// no callbacks required as this opens a popup which returns async
    	window.imagePicker.requestReadPermission();
  	}*/

  	function gotPhoto(imageUri) {
	    window.resolveLocalFileSystemURI(imageUri, 
	        function(fileEntry) {
	        	console.log(fileEntry);
	            fileEntry.file(function(fileObj) {
	                console.log("Size = " + fileObj.size);
	            },
	            function (error) {});
	        }, function (error) {}
	    );
	}

  	return {
    	handleMediaDialog: function saveMedia(type, id) {

    		var options,
    			deferred = $q.defer();
	      	
	      	if (type == 1)	
	      	{
	      		//from gallery
	      		console.log('gallery');
	      		options = {
			   		maximumImagesCount: 1,
			   		width: 1200,
			  	};

			  	//hasReadPermission();

			  	//console.log('window.imagePicker.getPictures');
			  	window.imagePicker.getPictures(
			    	function (success) {
				    	console.log(success);
				      	for (var i = 0; i < success.length; i++) {
				        	console.log('Image URI: ' + success[i]);
				      	}


				      	var imageUrl = success[0];
				      	//object with property nativeURL is expected
				      	//var ret = { nativeURL: success[0] };
				      	//deferred.resolve(ret);
				      	
				      	console.log(imageUrl);
				      	
				      	var name = imageUrl.substr(imageUrl.lastIndexOf('/') + 1);
				        var namePath = imageUrl.substr(0, imageUrl.lastIndexOf('/') + 1);
				        var newName = makeid() + name;

				        /*console.log(namePath);
				        console.log(name);
				        console.log(cordova.file.dataDirectory);
				        console.log(newName);
				        console.log('$cordovaFile.copyFile');*/
				        $cordovaFile.copyFile(namePath, name, cordova.file.dataDirectory, newName)
				        .then(function(success) {
				        	//console.log('$cordovaFile.copyFile success');
				        	console.log(success);

				        	//gotPhoto(success.nativeURL);

				            deferred.resolve(success);
				        }, function(error) {
				        	console.log(error);

				            deferred.reject(error);
				        });			      
			    	}, 
			    	function(error) {
				      	// error getting photos
				      	console.log(error);

				      	deferred.reject(error);
			    	},
			    options);

	      	} else
	      	{
	      		//from camera
	      		console.log('camera');
	      		options = optionsForType(type);

		      	$cordovaCamera.getPicture(options)
		      	.then(function(imageUrl) {
		      		console.log(imageUrl);
			        var name = imageUrl.substr(imageUrl.lastIndexOf('/') + 1);
			        var namePath = imageUrl.substr(0, imageUrl.lastIndexOf('/') + 1);
			        var newName = makeid() + name;

			        /*console.log(namePath);
			        console.log(name);
			        console.log(cordova.file.dataDirectory);
			        console.log(newName);*/

			        $cordovaFile.copyFile(namePath, name, cordova.file.dataDirectory, newName)
			        .then(function(success) {
			        	console.log(success);

			            deferred.resolve(success);
			        }, function(error) {
			        	CommonErrors.log(error);

			            deferred.reject(error);
			        });
		      	});
		    }

	      	return deferred.promise;	    
	  	}
  	};
})

.factory('ProFactory', function(localStorageService){

	return {
		get: function() {
	    	//get product cache
			var cache = localStorageService.get('pro_status');
			
	    	if (!angular.isDefined(cache))
	    		//no cache set, probably new install
				cache = false;	
			
           	return cache;	
	    },
	    set: function(status) {
			localStorageService.set('pro_status', status);
			
           	return true;	
	    },	    
	    getMessage: function() {
			//get product cache
			var cache = localStorageService.get('pro_status_message');
			
	    	if (!angular.isDefined(cache))
	    		//no cache set, probably new install
				cache = '';	
			
           	return cache;	
	    },	    
	    setMessage: function(msg) {
			localStorageService.set('pro_status_message', msg);
			
           	return true;	
	    },
	};
})

.factory('RatingsFilterFactory', function(localStorageService){

	var ratings_default = { 
						'name': [], 
						'brands': [], 
						'category': [],  
						'username': '',  
						'group': [],  
						'sort': 'recent', 
						'reviews': false, 
					};

	return {
		reset: function() {
	    	//reset finder cache
			//localStorageService.remove('filter_ratings_params');

			//save new cache
			localStorageService.set('filter_ratings_params', ratings_default);

           	return true;	
	    },
		getCache: function() {
	    	//get filter cache
			var cache = localStorageService.get('filter_ratings_params');
			
	    	if (!angular.isObject(cache))
	    		//no cache set, probably new install
				cache = ratings_default;	
			
           	return cache;	
	    },
	    setCache: function(field, value) {
	    	//console.log(field);
	    	//console.log(value);

			//get filter cache
			var cache = localStorageService.get('filter_ratings_params');

			if (!angular.isObject(cache))
				//no cache set, probably new install
				cache = ratings_default;		
			
			//add new filter to cache
			cache[field] = value;			

			//save new cache
			localStorageService.set('filter_ratings_params', cache);

		   	return this.getCache();
	    },
	    sort_options: function() {
			var obj = { 
						'recent': 'Most Recent',					
						'oldest': 'Oldest',					
						'highest': 'Highest Ratings',					
						'lowest': 'Lowest Ratings',					
						'helpful': 'Most Helpful',					
						'not_helpful': 'Least Helpful',					
						'funny': 'Funniest',					
						//'popular': 'Most Popular',														
					};

		   	return obj;
	    },
	    getfilterCount: function() {
			var ct = 0,
				filters = localStorageService.get('filter_ratings_params'),
				def = ratings_default;

			if (!angular.isObject(filters))
				return ct;

			ct = ct + filters.brands.length;
			ct = ct + filters.name.length;
			ct = ct + filters.category.length;

			if (filters.reviews)
				ct++;

			return ct;
		},
	};
})

.factory('FilterFactory', function(localStorageService){

	var default_params = { 
						'name': [], 
						'brands': [], 
						'category': [], 
						'iq_min': 0, 
						'iq_max': 100, 
						'price_min': 0, 
						'price_max': 100,  
						'review_min': 0,  
						'sort': 'relevance', 
						'sale': false, 
					};

	return {
		reset: function() {
	    	//reset finder cache
			localStorageService.remove('filter_params');

           	return true;	
	    },
		getCache: function() {
	    	//get filter cache
			var cache = localStorageService.get('filter_params');
			
	    	if (!angular.isObject(cache))
	    		//no cache set, probably new install
				cache = default_params;	
			
           	return cache;	
	    },
	    setCache: function(field, value) {
	    	//console.log(field);
	    	//console.log(value);

			//get filter cache
			var cache = localStorageService.get('filter_params');

			if (!angular.isObject(cache))
				//no cache set, probably new install
				cache = default_params;		
			
			//add new filter to cache
			cache[field] = value;			

			//save new cache
			localStorageService.set('filter_params', cache);

		   	return this.getCache();
	    },
	    getfilterCount: function() {
			var ct = 0,
				filters = localStorageService.get('filter_params'),
				def = default_params;

			if (!angular.isObject(filters))
				return ct;

			ct = ct + filters.brands.length;
			ct = ct + filters.name.length;
			ct = ct + filters.category.length;

			if (filters.iq_min != def.iq_min)
				ct++;

			if (filters.iq_max != def.iq_max)
				ct++;

			if (filters.price_min != def.price_min)
				ct++;

			if (filters.price_max != def.price_max)
				ct++;

			if (filters.review_min != def.review_min)
				ct++;

			if (filters.sale)
				ct++;

			return ct;
		},
	};
})

.factory('FinderFactory', function(localStorageService){

	var default_params = { 
						'name': [], 
						'brands': [], 
						'category': [], 
						'iq_min': 0, 
						'iq_max': 100, 
						'price_min': 0, 
						'price_max': 100, 
						'review_min': 0, 
						'sort': 'relevance', 
						'ingredients': [], 
						'notes': [], 
						'colors': [], 
						'retailers': [], 
						'attributes': [], 
						'concerns': [], 
						'sale': false, 
					};

	return {
		reset: function() {
	    	//reset finder cache
			localStorageService.set('finder_params', default_params);

           	return true;	
	    },
		getCache: function() {
	    	//get product cache
			var cache = localStorageService.get('finder_params');
			
	    	if (!angular.isObject(cache))
	    		//no cache set, probably new install
				cache = default_params;	
			
           	return cache;	
	    },
	    setCache: function(field, value) {
	    	//console.log(field);
	    	//console.log(value);

			//get product cache
			var cache = localStorageService.get('finder_params');

			if (!angular.isObject(cache))
				//no cache set, probably new install
				cache = default_params;		
			
			//add new product to cache
			cache[field] = value;			

			//save new cache
			localStorageService.set('finder_params', cache);

		   	return cache;
	    },
	    getfilterCount: function() {
			var ct = 0,
				filters = localStorageService.get('finder_params'),
				def = default_params;

			if (!angular.isObject(filters))
				return ct;

			ct = ct + filters.brands.length;
			ct = ct + filters.name.length;
			ct = ct + filters.category.length;
			ct = ct + filters.ingredients.length;
			ct = ct + filters.notes.length;
			ct = ct + filters.colors.length;
			ct = ct + filters.retailers.length;
			ct = ct + filters.attributes.length;
			ct = ct + filters.concerns.length;

			if (filters.iq_min != def.iq_min)
				ct++;

			if (filters.iq_max != def.iq_max)
				ct++;

			if (filters.price_min != def.price_min)
				ct++;

			if (filters.price_max != def.price_max)
				ct++;

			if (filters.review_min != def.review_min)
				ct++;

			if (filters.sale)
				ct++;

			return ct;
		},
	};
})

.factory('AssistantFactory', function(localStorageService){

	var default_params = { 
						'brands': [], 
						'brands_dislike': [], 
						'retailers': [], 
						'price': 'none', 
						'iq_min': 0, 
						'review_min': 0, 
						'ranks': false, 
						'ranks_overall': false, 
						'ingredients': [], 
						'ingredients_not': [], 
						'notes': [],  
						'attributes': [], 
						'concerns': [], 
						'colors_lip': [], 
						'colors_eye': [], 
						'colors_cheek': [], 
						'friends': [], 
						'groups': [], 
						'lists': false, 
						'strictness': 3, 
						'awards': false, 
						'mentions': false, 
					};

	return {
		reset: function() {
	    	//reset finder cache
			localStorageService.set('assistant_params', default_params);

           	return true;	
	    },
		getCache: function() {
	    	//get product cache
			var cache = localStorageService.get('assistant_params');
			
	    	if (!angular.isObject(cache))
	    		//no cache set, probably new install
				cache = default_params;	
			
           	return cache;	
	    },
	    setCache: function(field, value) {
			//get product cache
			var cache = localStorageService.get('assistant_params');

			if (!angular.isObject(cache))
				//no cache set, probably new install
				cache = default_params;		
			
			//add new product to cache
			cache[field] = value;			

			//save new cache
			localStorageService.set('assistant_params', cache);

		   	return cache;
	    },
	    setCacheAll: function(data) {	

	    	//database stores array as comma seperated strings. app expects arrays, so need to convert.
	    	var arrayFields = ['brands','brands_dislike','retailers','ingredients','ingredients_not','notes','attributes','concerns','colors_lip','colors_eye','colors_cheek','friends','groups'];

	    	angular.forEach(arrayFields, function(value)
	    	{
		      	if (data[value].indexOf(',') > -1)
		      	{
		      		//split string into array
		        	data[value] = data[value].split(',');

		        	//trim white spaces if any via: http://stackoverflow.com/questions/19293997/javascript-apply-trim-function-to-each-string-in-an-array
		        	data[value].map(Function.prototype.call, String.prototype.trim);
		      	} else if (data[value].length > 0)
		      	{
		      		//one item in list, convert to array
		      		var v = data[value];
		      			data[value] = [v];
		      	} else
		      		data[value] = [];
		   	});

		   	var boolFields = ['lists','mentions','ranks','ranks_overall','awards'];

		   	angular.forEach(boolFields, function(value)
		   	{
		      	if (data[value] === "true")
		      		data[value] = true;
		      	else
		      		data[value] = false;
		   	});

			//save new cache
			localStorageService.set('assistant_params', data);

		   	return data;
	    },
	    getfilterCount: function() {
			var ct = 0,
				filters = localStorageService.get('assistant_params'),
				def = default_params;

			if (!angular.isObject(filters))
				return ct;

			ct = ct + filters.brands.length;
			ct = ct + filters.name.length;
			ct = ct + filters.category.length;
			ct = ct + filters.ingredients.length;
			ct = ct + filters.notes.length;
			ct = ct + filters.colors.length;
			ct = ct + filters.retailers.length;
			ct = ct + filters.attributes.length;
			ct = ct + filters.concerns.length;

			if (filters.iq_min != def.iq_min)
				ct++;

			if (filters.iq_max != def.iq_max)
				ct++;

			if (filters.price_min != def.price_min)
				ct++;

			if (filters.price_max != def.price_max)
				ct++;

			if (filters.review_min != def.review_min)
				ct++;

			if (filters.sale)
				ct++;

			return ct;
		},
		wizardCompletion: function() {
			var ct = 0,
				fields = 8,
				filters = localStorageService.get('assistant_params'),
				def = default_params;

			if (!angular.isObject(filters))
				return ct;

			if (filters.brands.length !== 0)
				ct++;

			if (filters.iq_min !== 0)
				ct++;

			if (filters.price != def.price)
				ct++;

			if (filters.retailers.length !== 0)
				ct++;

			if (filters.attributes.length !== 0)
				ct++;

			if (filters.colors_lip.length !== 0)
				ct++;

			if (filters.concerns.length !== 0)
				ct++;

			if (filters.notes.length !== 0)
				ct++;
			
			return ct/fields * 100;
		},
	};
})

.factory('ReviewFinderFactory', function(localStorageService){

	var default_params = { 
						'text': '', 
						'helpful': '', 
						'not_helpful': '', 
						'funny': '', 
						'score_min': 1, 
						'score_max': 10, 
						'groups': [], 
						'retailers': [],  
						'sort': 'helpful', 
						'date_range': 'forever', 
						'verified': false, 
					};

	return {
		reset: function() {
	    	//reset finder cache
			localStorageService.set('review_finder_params', default_params);

           	return true;	
	    },
		getCache: function() {
	    	//get product cache
			var cache = localStorageService.get('review_finder_params');
			
	    	if (!angular.isObject(cache))
	    		//no cache set, probably new install
				cache = default_params;	
			
           	return cache;	
	    },
	    setCache: function(field, value) {
	    	//console.log(field);
	    	//console.log(value);

			//get product cache
			var cache = localStorageService.get('review_finder_params');

			if (!angular.isObject(cache))
				//no cache set, probably new install
				cache = default_params;		
			
			//add new product to cache
			cache[field] = value;			

			//save new cache
			localStorageService.set('review_finder_params', cache);

		   	return true;
	    },
	    getfilterCount: function() {
			var ct = 0,
				filters = localStorageService.get('review_finder_params'),
				def = default_params;

			if (!angular.isObject(filters))
				return ct;

			if (filters.text != def.text)
				ct++;

			if (filters.helpful != def.helpful)
				ct++;

			if (filters.not_helpful != def.not_helpful)
				ct++;

			if (filters.funny != def.funny)
				ct++;

			if (filters.score_min != def.score_min)
				ct++;

			if (filters.score_max != def.score_max)
				ct++;

			if (filters.date_range != def.date_range)
				ct++;

			if (filters.verified)
				ct++;

			return ct;
		},
	};
})

.factory('GlobalSettingsFactory', function(localStorageService){

	var default_params = { 
						'display_prices': true, 
					};

	return {
		reset: function() {
	    	//reset finder cache
			localStorageService.set('global_settings', default_params);

           	return true;	
	    },
		getCache: function() {
	    	//get product cache
			var cache = localStorageService.get('global_settings');
			
	    	if (!angular.isObject(cache))
	    		//no cache set, probably new install
				cache = default_params;	
			
           	return cache;	
	    },
	    setCache: function(field, value) {

			//get product cache
			var cache = localStorageService.get('global_settings');

			if (!angular.isObject(cache))
				//no cache set, probably new install
				cache = default_params;		
			
			//add new product to cache
			cache[field] = value;			

			//save new cache
			localStorageService.set('global_settings', cache);

		   	return this.getCache();
	    }
	};
})


.factory('AttributesOptionsFactory', function(){

	return {
		attributes: function() {
			var arr = [ 
						'organic',					
						'oil-free',					
						'gluten-free',					
						'paraben-free',					
						'cruelty-free',					
						'hypoallergenic',					
						'fragrance-free',					
						'non-comedogenic',														
					];

		   	return arr;
	    },
	    concerns: function() {
			var arr = [ 
						'acne',					
						'pores',					
						'redness',					
						'oiliness',					
						'dullness',					
						'wrinkles',					
						'puffiness',					
						'elasticity',					
						'dark spots',					
						'dark circles',										
					];

		   	return arr;
	    }, 
	    groups: function() {
			var arr = [ 
						'glam users',					
						'followers',					
						'following',										
						'makeup artist',					
						'hair stylist',					
						'dermatologist',					
						'esthetician',					
						'spa technician',					
						'publicist',					
						'brand manager',					
						'model',					
						'entertainer',					
						'blogger',										
						'journalist',										
					];

		   	return arr;
	    },
	    price_range: function() {
			var arr = { 
						'budget': '$ - Budget',															
						'prestige': '$$ - Prestige',															
						'luxury': '$$$ - Luxury',															
						'none': 'No Preference',															
					};

		   	return arr;
	    },
	    date_range: function() {
			var obj = { 
						'3days': 'Within last 3 days',					
						'week': 'This week',					
						'month': 'This month',					
						'year': 'This year',					
						'forever': 'Forever',														
					};

		   	return obj;
	    },
	    sort: function() {
			var obj = { 
						'relevance': 'Relevance',					
						'iq': 'Flauntme IQ',					
						'name': 'Name',					
						'price': 'Lowest Price',					
						'review_ct': 'Most Reviews',														
					};

		   	return obj;
	    },
	    review_sort: function() {
			var obj = { 
						'helpful': 'Most Helpful',					
						'not_helpful': 'Least Helpful',					
						'funny': 'Most Funny',					
						'newest': 'Newest',					
						'oldest': 'Oldest',					
						'highest': 'Score: Highest',														
						'lowest': 'Score: Lowest',														
					};

		   	return obj;
	    },
	    strictness: function() {
			var obj = { 
						'1': 'Not Very',					
						'3': 'Moderate',					
						'5': 'Very',																			
					};

		   	return obj;
	    },
	    colors_lip: function() {
			var obj = { 
						'Berry': '8f1e47',																		
						'Crimson': 'b5072c',																		
						'Pink': 'd03863',																		
						'Coral': 'ec3f45',																		
						'Brown': '9c552c',																		
						'Nude': 'ffcaa4',																		
						'Gold': '897238 ',																		
						'Silver': '959595',																		
						'Magenta': '630460',					
					};

		   	return obj;
	    }
	};
})

.factory('Notifications', function($ionicPopup){

	return {
		showPopup: function(title, body) {
			var alertPopup = $ionicPopup.alert({
		     	title: title,
		     	template: body,
		     	cssClass: 'alert-popup'
		   	});

		   	return alertPopup;
	    }
	};
})

.factory('TourFactory', function(localStorageService){

	//true means show the tour, false means already been seen so don't show

	var default_params = { 
						'assistant_recommendations': true, 
					};

	return {
		getCache: function(field) {
	    	//get filter cache
			var cache = localStorageService.get('tour_params');
			
	    	if (!angular.isObject(cache))
	    		//no cache set, probably new install
				cache = default_params;	
			
           	return cache[field];	
	    },
	    setCache: function(field, value) {
			//get filter cache
			var cache = localStorageService.get('tour_params');

			if (!angular.isObject(cache))
				//no cache set, probably new install
				cache = default_params;		
			
			//add new filter to cache
			cache[field] = value;			

			//save new cache
			localStorageService.set('tour_params', cache);

		   	return this.getCache();
	    }
	};
})

.factory('CommonErrors', function($rootScope, $cordovaToast, Notifications){

	return {
		check: function(data) {

			//check for valid json object
			if (typeof data != 'object')
			{
		        console.log('json issue');
                $rootScope.$broadcast('event:general-error');

                Notifications.showPopup('Alert','There was a server error. Please contact support if problems persist.');
                
                return;
		    }

			//check for old version
            if (data.type === 'version')
            {
            	console.log('version error');
                $rootScope.$broadcast('event:update-required');

                $cordovaToast.showLongBottom("Please update to the latest version of this app.");
                
                return;
            }

			//check for unauthorized message
            if (data.type === 'unauthorized')
            {
            	console.log('auth error');
                $rootScope.$broadcast('event:auth-login-required');

                $cordovaToast.showShortBottom("Please login.");

                return;
            }

            //check for limits reach message
            if (data.type === 'limit_reached')
            {
            	console.log('limit reached');
                $rootScope.$broadcast('event:max-limit-reached');

                Notifications.showPopup('Alert','Max usaged has been reached. Please upgrade your account.');

                //$cordovaToast.showLongBottom("Max usaged has been reached. Please upgrade your account.");

                return;
            }

            //check for pro accounts
            if (data.type === 'upgrade')
            {
                $rootScope.$broadcast('event:upgrade-required');

                Notifications.showPopup('Alert','Please upgrade your account to use this feature.');

                //$cordovaToast.showLongBottom("Please upgrade your account to use this feature.");

                return;
            }
	    },
	    log: function(data){
	    	console.log(data);
	    }
	};
})

;

