//http://stackoverflow.com/questions/21269007/interceptor-not-working
app.service('APIInterceptor', function($rootScope, $q, localStorageService) {
    return {
        request: function (config) {
            return config || $q.when(config);
        },
        requestError: function(request){
            return $q.reject(request);
        },
        response: function (response) {
            return response || $q.when(response);
        },
        responseError: function (response) {
            if (response.status === 401) {
                console.log('unauthorized');
                $rootScope.$broadcast('event:auth-login-required');
            }
            return $q.reject(response);
        }
    };
})

/*
    http://code.tutsplus.com/tutorials/token-based-authentication-with-angularjs-nodejs--cms-22543
    https://www.toptal.com/web/cookie-free-authentication-with-json-web-tokens-an-example-in-laravel-and-angularjs
    https://devdactic.com/user-auth-angularjs-ionic/
*/
.service('AuthenticationService', function($rootScope, $state, $http, Config, localStorageService, $ionicHistory, RequestService, Notifications, CommonErrors, ProFactory, $timeout) {
    var service = {
        login: function(username, password) {
            var data = {
                username: username,
                password: btoa(password),
                app: 'quoka',
                appVersion: Config.appVersion,
            };

            $http.post(Config.apiEndpoint+'checkIn', data)
            .success(function (data, status, headers, config) {                
                console.log('login success');
                
                // A more secure approach would be to store the token in SharedPreferences for Android, and Keychain for iOS
                localStorageService.set('auth_token', data._token);
                localStorageService.set('logged_in_user', data.user_data);
                ProFactory.set(data.pro_status);

                $rootScope.$broadcast('event:auth-login-success', status);

                $state.go('app.home');
            })
            .error(function (data, status, headers, config) {
                console.log('login error');
                $rootScope.$broadcast('event:auth-login-failed', status);

                Notifications.showPopup('Alert', data.message);
            });
        },
        logout: function() {
            var url = Config.apiEndpoint+"logOut";

            RequestService.getData(url)
            .finally(function(data) {
                console.log(data);

                //broadcast
                $rootScope.$broadcast('event:auth-logout-complete'); 
            });  

            this.clearUserData(); 

            //redirect
            $state.go('auth.login');         
        },
        clearUserData: function (){
            //clear user specific data/cache
            localStorageService.remove('auth_token');
            localStorageService.remove('logged_in_user');
            localStorageService.remove('cache_review_own');
            localStorageService.remove('cache_home');
            localStorageService.remove('cache_set');
            localStorageService.remove('cache_product');
            localStorageService.remove('cache_product_upc');
            localStorageService.remove('cache_categories');
            localStorageService.remove('cache_review_extreme');
            localStorageService.remove('cache_brand');
            localStorageService.remove('cache_brand_products');
            localStorageService.remove('cache_brand_followers');
            localStorageService.remove('cache_blog_index');
            localStorageService.remove('cache_blog_post');
            localStorageService.remove('cache_review_temp');
            localStorageService.remove('cache_review_image');
            localStorageService.remove('pro_status');
            localStorageService.remove('assistant_params');
            localStorageService.remove('filter_params');
            localStorageService.remove('finder_params');

            //clear view history
            $timeout(function () {
                $ionicHistory.clearCache();
                $ionicHistory.clearHistory();
                console.log('clearing view history');
            },300);
        },
        forgot: function(username, email) {
            var data = {
                username: username,
                email: btoa(email),
                app: 'quoka',
                appVersion: Config.appVersion,
            };

            $http.post(Config.apiEndpoint+'forgotPassword', data)
            .then(function (data, status, headers, config) {                
                console.log(data);                                

                $rootScope.$broadcast('event:forgot-password-success', status);
                
                //success message
                var popup = Notifications.showPopup('Success', data.data.message);
            
                //after confirm button press
                popup.then(function() {
                    //go to login page             
                    $state.go('auth.login');
                });

            }, function (data, status, headers, config) {
                console.log(data);            
                $rootScope.$broadcast('event:forgot-password-failed', status);

                Notifications.showPopup('Alert', data.data.message);
            });
        },
        register: function(user) {
            var data = {
                username: user.username,
                password: btoa(user.password),
                email: btoa(user.email),
                app: 'quoka',
                appVersion: Config.appVersion,
            };

            $http.post(Config.apiEndpoint+'newAccount', data)
            .success(function (data, status, headers, config) {                
                console.log(data);                                

                $rootScope.$broadcast('event:new-account-success', status);

                localStorageService.set('auth_token', data._token);
                localStorageService.set('logged_in_user', data.user_data);

                //success message
                var popup = Notifications.showPopup('Success', data.message);
            
                //after confirm button press
                popup.then(function() {
                    //go to home page             
                    $state.go('app.home');
                });                
            })
            .error(function (data, status, headers, config) {
                console.log(data);            
                $rootScope.$broadcast('event:new-account-failed', status);
                
                if (data.message)
                    Notifications.showPopup('Alert', data.message);
                else
                    Notifications.showPopup('Alert', 'There was an error with your signup. Please try again or register via our website.');
            });
        }
    };

    return service;
})

.service('ExternalLoginService', function($rootScope, $window, $state, localStorageService, Config){

    var url, loginWindow, token, hasToken, userId, hasUserId;

    return {
        google: function (){
            console.log('google'); 
            url = Config.socialLoginEndpoint+"social_login/google?redirect=mobile";
            loginWindow = $window.open(url, '_blank', 'location=no,toolbar=no');
            //loginWindow = $window.open(url, '_blank', '');
            loginWindow.addEventListener('loadstart', function(event){
                console.log('listener');                
                hasToken = event.url.indexOf('?oauth_token=');                

                if (hasToken > -1) {
                    //store token
                    token = event.url.match('oauth_token=(.*)&userId')[1];
                    localStorageService.set('auth_token', token);

                    /*
                        As for the user meta, we store the user id + username temporarily in localstorage.
                        This also acts as flag to get it on the next page load.
                    */
                    //var user_data = {};
                    //user_data.member_id = event.url.match('userId=(.*)&username')[1];
                    //user_data.username = event.url.match('username=(.*)&end=')[1];
                    //localStorageService.set('temp_user_data', user_data);

                    //close external login window
                    loginWindow.close();
                    location.href=location.pathname;

                    //broadcast event
                    $rootScope.$broadcast('event:social-account-paired');
                    $rootScope.$broadcast('event:auth-login-success');

                    loginWindow.removeEventListener('loadstart', function(event){});

                    $state.go('app.home');
                }
            });

        }
    };

})

.service("UserService", function($q, Config, RequestService, localStorageService, ProFactory, RatingsFilterFactory, CommonErrors, $cordovaFileTransfer, $ionicLoading, $ionicPlatform) {

    var checkCanopy = function(user){
            //console.log(user);
            //if no canopy, show random one based on user id
            if (!angular.isDefined(user.canopy))
            {
                var rand = user.member_id % 10;
                user.canopy = 'img/default/default-canopy-'+rand+'.jpg';
            }

            return user;
        };

    return {
        clearCache: function() {
            console.log('clearCache');
            user = localStorageService.remove('logged_in_user');
            
            return;
        },
        setCache: function(field, value) {
            console.log('setCache');
            console.log(field);
            console.log(value);
            var user = localStorageService.get('logged_in_user');

            user[field] = value;

            localStorageService.set('logged_in_user', user);
            
            return user;
        },
        getLoggedInUser: function() {
            var url = Config.apiEndpoint+"userMeta",
                user = localStorageService.get('logged_in_user');
            
            if (user === null)
            {
                RequestService.getData(url)
                .then(function(success){
                    console.log(success);

                    success.user_data = checkCanopy(success.user_data);

                    //store in cache
                    localStorageService.set('logged_in_user', success.user_data);
                    ProFactory.set(success.pro_status);

                    user = success.user_data; 
                },function(error){                    
                    return {};
                });
            } else
            {
                //cache exist, check canopy
                user = checkCanopy(user);
            }

            return user;
        },      
        getProStatus: function() {
            var deferred = $q.defer(),
                url = Config.apiEndpoint+"userProStatus";

            RequestService.getData(url)
            .then(function(success){
                console.log(success);

                //store in cache
                ProFactory.set(success.ret);

                deferred.resolve(success.ret); 
            },function(error){                    
                deferred.reject(error);
            }); 
            
            return deferred.promise;   
        },
        search: function(query) {
            var url = Config.apiEndpoint+"userSearch",
                data = {'query': query};

            return RequestService.getData(url, data);
        },
        profile: function(username) {
            var url = Config.apiEndpoint+"userProfile",
                data = {'username': username};

            return RequestService.getData(url, data);
        },
        toggleFollow: function(username) {
            var deferred = $q.defer(),
                url = Config.apiEndpoint+"userFollow",
                data = { 'username': username };

            RequestService
                .getData(url, data)
                .then(function(success){

                    deferred.resolve(success);
                }, function(error){
                    deferred.reject(error);
                });

            return deferred.promise;
        },
        ratings: function(user, page) {
            var pos = typeof page !== 'undefined' ? page : 1,
                data = RatingsFilterFactory.getCache(),
                url = Config.apiEndpoint+"userRatings/"+user+"?page="+pos;

            return RequestService.getData(url, data);
        },
        ratingsByProduct: function(id, page) {
            var pos = typeof page !== 'undefined' ? page : 1,
                data = RatingsFilterFactory.getCache(),
                url = Config.apiEndpoint+"reviewSearchInternal/"+id+"?page="+pos;            

            return RequestService.getData(url, data);
        },
        review: function(id) {
            var url = Config.apiEndpoint+"userReview/"+id;

            return RequestService.getData(url);
        },
        followers: function(user, page) {
            var pos = typeof page !== 'undefined' ? page : 1;
            
            var url = Config.apiEndpoint+"userFollowers/"+user+"?page="+pos;

            return RequestService.getData(url);
        },
        following: function(user, page) {
            var pos = typeof page !== 'undefined' ? page : 1;
            
            var url = Config.apiEndpoint+"userFollowing/"+user+"?page="+pos;

            return RequestService.getData(url);
        },
        account: function() {
            var url = Config.apiEndpoint+"account";

            return RequestService.getData(url);
        },
        accountUpdate: function(data) {
            var url = Config.apiEndpoint+"accountUpdate";

            return RequestService.getData(url, data);
        },
        accountEmail: function() {
            var url = Config.apiEndpoint+"accountEmail";

            return RequestService.getData(url);
        },
        accountEmailUpdate: function(data) {
            var url = Config.apiEndpoint+"accountEmailUpdate",
                send = { 'email': btoa(data.email), 'password': btoa(data.password) };

            return RequestService.getData(url, send);
        },
        accountPasswordUpdate: function(data) {
            var url = Config.apiEndpoint+"accountPasswordUpdate",
                send = { 'old': btoa(data.old), 'password': btoa(data.password) };

            return RequestService.getData(url, send);
        },
        userPhotoUpload: function(data)
        {
            //add required info
            data.Authorization = localStorageService.get('auth_token');
            data.appVersion = Config.appVersion;                    

            var deferred = $q.defer(),
                url = Config.apiEndpoint+"userPhotoUpload",
                name = data.photo.substr(data.photo.lastIndexOf('/') + 1),
                //file = new File(data.photo),
                options = {
                    fileKey: "userfile",
                    fileName: "user_photo.jpg",
                    chunkedMode: false,
                    mimeType: "image/jpg",
                    params: data
                };

            $ionicLoading.show({template: '<img src="img/preloader-heart-accent.gif" />'});

            $ionicPlatform.ready(function() {

                $cordovaFileTransfer.upload(url, data.photo, options)
                .then(function(success){
                    console.log('file upload success');
                    console.log(success);

                    deferred.resolve(success);
                },function(error){
                    console.log('file upload error');
                    CommonErrors.log(error);

                    deferred.reject(error.body);
                }).finally(function(){
                    $ionicLoading.hide();
                });
                
            });

            return deferred.promise;
        },
    };
})

.service("BarcodeService", function($q, Config, $cordovaBarcodeScanner, RequestService, cacheJSON) {
    return {
        initScan: function(){
            var deferred = $q.defer();

            $cordovaBarcodeScanner
            .scan()
            .then(function(success) {                               
                console.log(success);   

                //check if upc is in cache
                var cache = cacheJSON.getCache(success.text, 'product_upc'); 

                if (cache !== false)
                    //yes, attached product_id to return object
                    success.product_id = cache;

                deferred.resolve(success);          
            }, function(error) {
                deferred.reject(error);
                console.log("An error happened -> " + error);
            });

            return deferred.promise;
        },
        submit: function(data) {            
            var url = Config.apiEndpoint+"barcodeSubmit";

            return RequestService.getData(url, data);
        }
    };
})

.service("OCRService", function($q, $http, Config, $cordovaFileTransfer, $ionicLoading, RequestService) {
    return {
        search: function(str) {
            //strip returns+newlines
            str = str.replace(/[\n\r]/g, ' ');

            var url = Config.apiEndpoint+"OCRSearch",
                data = { key: btoa(unescape(encodeURIComponent(str))) };

            return RequestService.getData(url, data);
        },
        send: function(image) {            
            var deferred = $q.defer(),
                data = {
                    apikey: Config.ocr_api_key,
                    language: 'eng',
                },
                url = "https://api.ocr.space/parse/image";

            //loading screen
            $ionicLoading.show({template: '<img src="img/preloader-heart-accent.gif" />'});

            /*var formData = new FormData();
            formData.append("file", fileToUpload);
            formData.append("url", "URL-of-Image-or-PDF-file");
            formData.append("language", "eng");
            formData.append("apikey", "Your-API-Key-Here");

            formData.append("isOverlayRequired", True);*/


            var options = {
                fileKey: "avatar",
                fileName: "text.png",
                chunkedMode: false,
                mimeType: "image/png",
                params: data
            };

            $cordovaFileTransfer.upload(url, "/img/ocr/text.png", options).then(function(result) {
                console.log("SUCCESS: " + JSON.stringify(result.response));
                deferred.resolve(data);
            }, function(err) {
                console.log("ERROR: " + JSON.stringify(err));
                deferred.reject(data);
            }, function (progress) {
                // constant progress updates
            });

            /*$http.post(url, data)
            .success(function(data) {
                deferred.resolve(data);
            })
            .error(function(data) {                                
                deferred.reject(data);                
            }).finally(function(){
                //remove loading screen
                $ionicLoading.hide();
            });  */ 

            return deferred.promise; 
        }
    };
})

.service("ProductService", function($q, Config, RequestService, cacheJSON) {

    return {
        setCache: function(productID, product) {
            var cache = cacheJSON.getCache(productID, 'product');
           
            cacheJSON.setCache(productID, product, 'product');

            return cache;                         
        },
        setProduct: function(productObj){
            var ProductService = this;
            ProductService.product = productObj;
            return this.product;
        },
        getProduct: function(){
            var ProductService = this;
            return this.product;
        },
        getProductByUPC: function(productUPC) {
            var deferred = $q.defer(),
                url = Config.apiEndpoint+"getProductFromUPC/"+productUPC;

            RequestService.getData(url).then(function(success){

                //check to see if unknown came back
                if (success.ret !== false)
                    //add to upc cache
                    cacheJSON.setCache(productUPC, success.id, 'product_upc');
                
                deferred.resolve(success);
            }, function(error){
                deferred.reject(error);
            });

            return deferred.promise; 
        },
        getProductByID: function(productID) {
            var cache = cacheJSON.getCache(productID, 'product');
          
            //do we have a cache of this set?
            if (cache !== false)
                //yes, just return that
                return $q.when(cache);
           
            //cache is too old or nonexistant, grab new
            var deferred = $q.defer(),
                url = Config.apiEndpoint+"getProduct/"+productID;

            RequestService.getData(url).then(function(success){
                //add to cache
                cacheJSON.setCache(productID, success, 'product');
                
                deferred.resolve(success);
            }, function(error){
                deferred.reject(error);
            });

            return deferred.promise;                         
        },
        getPrices: function(id) {
            var url = Config.apiEndpoint+"priceList/"+id;

            return RequestService.getData(url);
        },
        saveRating: function(id, rating) {
            var url = Config.apiEndpoint+"saveRating",
                data = {
                    rating: rating,
                    id: id
                };

            return RequestService.getData(url, data);
        },
        getSet: function(id) {
            var cache = cacheJSON.getCache(id, 'set');
            
            //do we have a cache of this set?
            if (cache !== false)
                //yes, just return that
                return $q.when(cache);

            var url = Config.apiEndpoint+"getSet/"+id;

            return RequestService.getData(url);
        },
        getList: function(id, page) {
            var pos = typeof page !== 'undefined' ? page : 1;
            
            if (pos === 1)
            {
                var cache = cacheJSON.getCache(id, 'list');

                //do we have a cache of this set?
                if (cache !== false)
                    //yes, just return that
                    return $q.when(cache);
            }

            var url = Config.apiEndpoint+"getList/"+id+"?page="+pos;

            //we don't know the list id of special list, so we get it on the first call
            if (id === 'heart' && pos == 1)
                url = Config.apiEndpoint+"listSpecial?type=heart";

            return RequestService.getData(url);
        },
        getCategoryList: function() {
            var cache = cacheJSON.getCache(1, 'category_tree');
            
            //do we have a cache of this set?
            if (cache !== false)
                //yes, just return that
                return $q.when(cache);

            var deferred = $q.defer(),
                url = Config.apiEndpoint+"getCategoryTree";

            RequestService.getData(url).then(function(success){
                //add to cache
                cacheJSON.setCache(1, success, 'category_tree');

                deferred.resolve(success);
            }, function(error){
                deferred.reject(error);
            });

            return deferred.promise;
        },
    };
})

.service("ReviewService", function(Config, RequestService, cacheJSON, $q, $cordovaFileTransfer, ReviewFinderFactory, CommonErrors, localStorageService, $ionicPlatform, $ionicLoading, $cordovaToast, $cordovaFile, Upload) {

    var submitReview = function(data)
        {
            var deferred = $q.defer(),
                url = Config.apiEndpoint+"reviewSubmit";

            RequestService.getData(url, data)
            .then(function(success){
                console.log('post success');
                //update product cache to show pending review
                var cache = cacheJSON.getCache(data.id, 'product');
                //update status
                cache.user_rating_status = 'pending';
                //save
                cacheJSON.setCache(data.id, cache, 'product', true);

                deferred.resolve(success);               
            }, function(error){
                //error
                console.log('post error');
                CommonErrors.log(error);
                deferred.reject(error);
            });

            return deferred.promise;
        },
        submitPhoto = function(data)
        {
            //add required info
            data.Authorization = localStorageService.get('auth_token');
            data.appVersion = Config.appVersion;                    

            var deferred = $q.defer(),
                url = Config.apiEndpoint+"reviewSubmit",
                name = data.photo.substr(data.photo.lastIndexOf('/') + 1),
                //file = new File(data.photo),
                options = {
                    fileKey: "userfile",
                    fileName: "user_review.jpg",
                    chunkedMode: false,
                    mimeType: "image/jpg",
                    params: data
                };

            //console.log(file);

            $ionicLoading.show({template: '<img src="img/preloader-heart-accent.gif" />'});

            $ionicPlatform.ready(function() {

                $cordovaFileTransfer.upload(url, encodeURI(data.photo), options)
                .then(function(success){
                    console.log('file upload success');
                    console.log(success);

                    deferred.resolve(success);
                },function(error){
                    console.log('file upload error');
                    CommonErrors.log(error);

                    deferred.reject(error.body);
                }).finally(function(){
                    $ionicLoading.hide();
                });
                
            });

            return deferred.promise;
        };

    return {
        getReviewExtreme: function(id) {
            var deferred = $q.defer(),
                cache = cacheJSON.getCache(id, 'review_extreme');
            
            //do we have a cache of this?
            if (cache !== false)
                //yes, just return that
                return $q.when(cache);

            var url = Config.apiEndpoint+"getReviewExtreme/"+id;                

            RequestService.getData(url)
            .then(function(success){
                //save to cache
                cacheJSON.setCache(id, success, 'review_extreme', true);

                deferred.resolve(success);
            }, function(error){
                //error
                deferred.reject(error);
            });

            return deferred.promise;
        },
        submit: function(data){  

            var deferred = $q.defer();

            //is there a photo to submit?                                  
            if (data.photo !== '')
            {
                //yes, submit photo first + review
                submitPhoto(data)
                .then(function(success){                    
                    deferred.resolve(success);
                }, function(error){
                    CommonErrors.log(error);
                    deferred.reject(error); 
                });
            } else
            {
                //no, just submit review
                submitReview(data)
                .then(function(success){
                    deferred.resolve(success);
                },function(error){
                    deferred.reject(error); 
                });
            }

            return deferred.promise;
        },
        save: function(id, data){            
            cacheJSON.setCache(id, data, 'review_temp', true);

            return true;            
        },
        getTempReview: function(id)
        {
            var cache = cacheJSON.getCache(id, 'review_temp');

            if (cache !== false)
                return cache;
            else
                return {title: '', review: '', photo: ''};
        },
        getOwnReview: function(id)
        {
            var deferred = $q.defer(),
                cache = cacheJSON.getCache(id, 'review_own');
            
            //do we have a cache of this?
            if (cache !== false)
                //yes, just return that
                return $q.when(cache);

            var url = Config.apiEndpoint+"getReviewOwn/"+id;                

            RequestService.getData(url)
            .then(function(success){
                //save to cache
                cacheJSON.setCache(id, success, 'review_own', true);                

                deferred.resolve(success);
            }, function(error){
                //error
                deferred.reject(error);
            });

            return deferred.promise;
        },
        search: function(data) {
            var url = Config.apiEndpoint+"reviewFinderSearch";

            //remove options property
            data.options = '';

            return RequestService.getData(url, data);
        },
        query: function(id, page) {
            var pos = typeof page !== 'undefined' ? page : 1,
                data = ReviewFinderFactory.getCache(),
                url = Config.apiEndpoint+"reviewSearch/"+id+"?page="+pos;            

            return RequestService.getData(url, data);
        },
        vote: function(id, action) {
            var url = Config.apiEndpoint+"reviewVote/"+id+"?action="+action;            

            return RequestService.getData(url);
        }
    };
})

.service("HomeService", function($q, Config, RequestService, cacheJSON) {

    return {
        home: function() {
            var cache = cacheJSON.getCache(1, 'home');
            
            //do we have a cache of this set?
            if (cache !== false)
                //yes, just return that
                return $q.when(cache);

            var deferred = $q.defer(),
                url = Config.apiEndpoint+"homev2";

            RequestService.getData(url).then(function(success){
                //add to cache
                cacheJSON.setCache(1, success, 'home');

                deferred.resolve(success);
            }, function(error){
                deferred.reject(error);
            });

            return deferred.promise;
        },
        getHomeLists: function() {
            var cache = cacheJSON.getCache(1, 'home_list');
            
            //do we have a cache of this set?
            if (cache !== false)
                //yes, just return that
                return $q.when(cache);

            var deferred = $q.defer(),
                url = Config.apiEndpoint+"homeList";

            RequestService.getData(url).then(function(success){
                //add to cache
                cacheJSON.setCache(1, success, 'home_list');

                deferred.resolve(success);
            }, function(error){
                deferred.reject(error);
            });

            return deferred.promise;
        },
        getPopularProducts: function() {
            var cache = cacheJSON.getCache(1, 'popular_products');
            
            //do we have a cache of this set?
            if (cache !== false)
                //yes, just return that
                return $q.when(cache);

            var deferred = $q.defer(),
                url = Config.apiEndpoint+"homePopularProducts";

            RequestService.getData(url).then(function(success){
                //add to cache
                cacheJSON.setCache(1, success, 'popular_products');

                deferred.resolve(success);
            }, function(error){
                deferred.reject(error);
            });

            return deferred.promise;
        },
        getPopularBrands: function() {
            var cache = cacheJSON.getCache(1, 'popular_brands');
            
            //do we have a cache of this set?
            if (cache !== false)
                //yes, just return that
                return $q.when(cache);

            var deferred = $q.defer(),
                url = Config.apiEndpoint+"homePopularBrands";

            RequestService.getData(url).then(function(success){
                //add to cache
                cacheJSON.setCache(1, success, 'popular_brands');

                deferred.resolve(success);
            }, function(error){
                deferred.reject(error);
            });

            return deferred.promise;
        }
    };
})

.service("BlogService", function(Config, RequestService, cacheJSON, $q) {

    return {
        getBlogIndex: function(page) {
            var pos = typeof page !== 'undefined' ? page : 1;

            if (pos === 1)
            {                
                //we only cache the first page
                cache = cacheJSON.getCache(1, 'blog_index');
                        
                //do we have a cache of this set?
                if (cache !== false)
                    //yes, just return that
                    return $q.when(cache);
            }

            var deferred = $q.defer(),
                url = Config.apiEndpoint+"blogIndex?page="+pos;

            RequestService.getData(url).then(function(success){
                if (pos === 1)
                    //add to cache
                    cacheJSON.setCache(1, success, 'blog_index');

                deferred.resolve(success);
            }, function(error){
                deferred.reject(error);
            });

            return deferred.promise;
        },
        getPost: function(postId) {
            var cache = cacheJSON.getCache(postId, 'blog_post');
            
            //do we have a cache of this?
            if (cache !== false)
                //yes, just return that
                return $q.when(cache);

            var deferred = $q.defer(),
                url = Config.apiEndpoint+"getBlogPost/"+postId;

            RequestService.getData(url).then(function(success){
                //add to cache
                cacheJSON.setCache(postId, success, 'blog_post');

                deferred.resolve(success);
            }, function(error){
                deferred.reject(error);
            });

            return deferred.promise;
        }
    };
})

.service("SearchService", function(Config, RequestService) {

    return {
        searchByName: function(name) {
            var url = Config.apiEndpoint+"searchByName/"+name;

            return RequestService.getData(url);
        }
    };
})

.service("BrandService", function(Config, RequestService, cacheJSON, $q) {

    return {
        brandByID: function(id) {
            var cache = cacheJSON.getCache(id, 'brand');
            
            //do we have a cache of this set?
            if (cache !== false)
                //yes, just return that
                return $q.when(cache);

            var url = Config.apiEndpoint+"brand/"+id;

            return RequestService.getData(url);
        },
        getProducts: function(id, page) {
            var pos = typeof page !== 'undefined' ? page : 1;
            
            if (pos === 1)
            {                
                //we only cache the first page
                cache = cacheJSON.getCache(id, 'brand_products');
                        
                //do we have a cache of this set?
                if (cache !== false)
                    //yes, just return that
                    return $q.when(cache);
            }

            var deferred = $q.defer(),
                url = Config.apiEndpoint+"brandProducts/"+id+"?page="+pos;

            RequestService.getData(url).then(function(success){
                if (pos === 1)
                    //add to cache
                    cacheJSON.setCache(id, success, 'brand_products');

                deferred.resolve(success);
            }, function(error){
                deferred.reject(error);
            });

            return deferred.promise;
        },
        getFollowers: function(id, page) {
            var pos = typeof page !== 'undefined' ? page : 1;
            
            if (pos === 1)
            {                
                //we only cache the first page
                cache = cacheJSON.getCache(id, 'brand_followers');
                        
                //do we have a cache of this set?
                if (cache !== false)
                    //yes, just return that
                    return $q.when(cache);
            }

            var deferred = $q.defer(),
                url = Config.apiEndpoint+"brandFollowers/"+id+"?page="+pos;

            RequestService.getData(url).then(function(success){
                if (pos === 1)
                    //add to cache
                    cacheJSON.setCache(id, success, 'brand_followers');

                deferred.resolve(success);
            }, function(error){
                deferred.reject(error);
            });

            return deferred.promise;
        },
        toggleFollow: function(bid) {
            var deferred = $q.defer(),
                url = Config.apiEndpoint+"brandToggleFollow/"+bid;

            RequestService.getData(url).then(function(success){
                //wipe cache 
                cacheJSON.unsetCache(bid, 'brand_followers');

                deferred.resolve(success);
            }, function(error){
                deferred.reject(error);
            });

            return deferred.promise;
        }
    };
})

.service("CategoryService", function(Config, RequestService, cacheJSON, $q) {

    return {
        searchByID: function(id, page) {
            var pos = typeof page !== 'undefined' ? page : 1;
            
            if (pos === 1)
            {              
                //we only cache the first page
                cache = cacheJSON.getCache(id, 'categories');
                       
                //do we have a cache of this set?
                if (cache !== false)
                    //yes, just return that
                    return $q.when(cache);
            }

            var deferred = $q.defer(),
                url = Config.apiEndpoint+"getProductByCategory/"+id+"?page="+pos;

            RequestService.getData(url).then(function(success){
                if (pos === 1)
                    //add to cache
                    cacheJSON.setCache(id, success, 'categories');

                deferred.resolve(success);
            }, function(error){
                deferred.reject(error);
            });

            return deferred.promise;
        },
        getHome: function(id) {            
            var url = Config.apiEndpoint+"categoryHome/"+id;

            return RequestService.getData(url);
        }
    };
})

.service("HeartService", function(Config, RequestService, $q, cacheJSON) {

    return {
        toggleHeart: function(mid) {
            var deferred = $q.defer(),
                url = Config.apiEndpoint+"toggleHeart",
                data = {
                    set: 'products',
                    type: 'heart',
                    media_id: mid
                };
     
            RequestService.getData(url, data, false).then(function(success){
                //update the heart field in cache if exist

                //get cache
                var cache = cacheJSON.getCache(mid, 'product');
                console.log(cache);
                if (cache !== false)
                {   console.log(cache.like_state);
                    if (cache.like_state === 0)
                        cache.like_state = 1;
                    else
                        cache.like_state = 0;

                    //save new cache
                    cacheJSON.setCache(mid, cache, 'product', true);
                }

                deferred.resolve(success);
            }, function(error){
                deferred.reject(error);
            });

            return deferred.promise;    
        },
        showHeartList: function() {
            var url = Config.apiEndpoint+"heartList";

            return RequestService.getData(url);
        }
    };
})

.service("CommentService", function(Config, RequestService) {

    return {
        submit: function(data) {
            var url = Config.apiEndpoint+"commentSubmit";

            return RequestService.getData(url, data);
        },
        get: function(data, page) {
            var pos = typeof page !== 'undefined' ? page : 1,
                url = Config.apiEndpoint+"commentGet/"+data.id+'?page='+pos;

            return RequestService.getData(url, data);
        }
    };
})

.service("AssistantService", function(Config, RequestService, localStorageService) {

    return {
        get: function() {
            var url = Config.apiEndpoint+"assistantGet";

            return RequestService.getData(url);
        },
        save: function(data) {
            var url = Config.apiEndpoint+"assistantSave";

            return RequestService.getData(url, data);
        },
        saveAll: function() {
            var url = Config.apiEndpoint+"assistantSaveAll",
                data = localStorageService.get('assistant_params');

            return RequestService.getData(url, data);
        },
        search: function(data) {
            var url = Config.apiEndpoint+"assistantSearch";

            //remove options property
            data.options = '';

            return RequestService.getData(url, data);
        },
        reset: function() {
            var url = Config.apiEndpoint+"assistantReset";

            return RequestService.getData(url);
        },
        getRecs: function() {
            var url = Config.apiEndpoint+"assistantRecommendations";

            return RequestService.getData(url);
        },
        saveFeedback: function(status, pid) {
            var url = Config.apiEndpoint+"assistantSaveFeedback",
                data = { 'status': status, 'pid': pid };

            return RequestService.getData(url, data);
        },
        getHistory: function(state, page) {
            var pos = typeof page !== 'undefined' ? page : 1,
                data = { 'state': state },
                url = Config.apiEndpoint+"assistantHistory?page="+pos;            

            return RequestService.getData(url, data);
        },
    };
})

.service("FinderService", function(Config, RequestService, FinderFactory, FilterFactory) {

    return {
        search: function(data) {
            var url = Config.apiEndpoint+"finderSearch";

            //remove options property
            data.options = '';

            return RequestService.getData(url, data);
        },
        query: function(page) {
            var pos = typeof page !== 'undefined' ? page : 1,
                data = FinderFactory.getCache(),
                url = Config.apiEndpoint+"finder?page="+pos;            

            return RequestService.getData(url, data);
        },
        queryFilter: function(page) {
            var pos = typeof page !== 'undefined' ? page : 1,
                data = FilterFactory.getCache(),
                url = Config.apiEndpoint+"finder?page="+pos;            

            return RequestService.getData(url, data);
        }
    };
})

.service("ProService", function(Config, RequestService) {

    return {
        subscribe: function(data) {
            var url = Config.apiEndpoint+"proSubscribe";

            //add platform
            data.platform = ionic.Platform.isIOS() ? 'ios-appstore' : 'android-playstore';
            //data.platform = 'ios-appstore';

            return RequestService.getData(url, data);
        }
    };
})

.service("RequestService", function($http, $q, Config, localStorageService, $ionicLoading, CommonErrors) {

    return {
        getData: function(url, data, loading) {
            var deferred = $q.defer(),
                doLoading = typeof loading !== 'undefined' ? loading : true;

            //attach auth token if present
            if (typeof data !== 'undefined')
            {
                //data object exist, add auth field
                data.Authorization = localStorageService.get('auth_token');   
                data.appVersion = Config.appVersion;             
            } else
            {
                //no data object, set authorization
                data = {'Authorization': localStorageService.get('auth_token'), 'appVersion': Config.appVersion};
            }

            //loading screen
            if (doLoading)
                $ionicLoading.show({template: '<img src="img/preloader-heart-accent.gif" />'});

            $http.post(url, data)
            .success(function(data) {
                CommonErrors.check(data);

                deferred.resolve(data);
            })
            .error(function(data) {    
                CommonErrors.check(data);
                                           
                deferred.reject(data);                
            }).finally(function(){
                //remove loading screen if requested
                if (doLoading)
                    $ionicLoading.hide();
            });

            return deferred.promise;
        }
    };
})

;